package eu.eudat.controllers;

import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.transaction.Transactional;

import eu.eudat.entities.DatasetProfile;
import eu.eudat.entities.DatasetProfileRuleset;
import eu.eudat.entities.DatasetProfileViewstyle;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import eu.eudat.dao.entities.DMPDao;
import eu.eudat.dao.entities.DMPProfileDao;
import eu.eudat.dao.entities.DataRepositoryDao;
import eu.eudat.dao.entities.DatasetDao;
import eu.eudat.dao.entities.DatasetProfileDao;
import eu.eudat.dao.entities.DatasetProfileRulesetDao;
import eu.eudat.dao.entities.DatasetProfileViewstyleDao;
import eu.eudat.dao.entities.OrganisationDao;
import eu.eudat.dao.entities.ProjectDao;
import eu.eudat.dao.entities.RegistryDao;
import eu.eudat.dao.entities.ResearcherDao;
import eu.eudat.dao.entities.ServiceDao;


@RestController
@CrossOrigin
public class DatasetProfiles  {
	
	@Autowired private DataRepositoryDao dataRepositoryDao;
	@Autowired private DatasetDao datasetDao;
	@Autowired private DatasetProfileDao datasetProfileDao;
	@Autowired private DatasetProfileRulesetDao datasetProfileRulesetDao;
	@Autowired private DatasetProfileViewstyleDao datasetProfileViewstyleDao;
	@Autowired private DMPDao dMPDao;
	@Autowired private DMPProfileDao dMPProfileDao;
	@Autowired private OrganisationDao organisationDao;
	@Autowired private ProjectDao projectDao;
	@Autowired private RegistryDao registryDao;
	@Autowired private ResearcherDao researcherDao;
	@Autowired private ServiceDao serviceDao;
	
	
	//FETCH BY DATASET PROFILE
	
	@RequestMapping(method = RequestMethod.GET, value = { "/datasetprofiles" }, produces="text/plain")
	public @ResponseBody ResponseEntity<List<UUID>> listDMPs(){
		try {
			List<UUID> allIDs = datasetProfileDao.listAllIDs();
			return ResponseEntity.status(HttpStatus.OK).body(allIDs);
		}
		catch(Exception ex) {
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
		}
	}
	
	
	@RequestMapping(method = RequestMethod.GET, value = { "/datasetprofiles/{id}" })
	public @ResponseBody ResponseEntity<DatasetProfile> getDatasetProfile(@PathVariable("id") String id) {
		try {
			DatasetProfile profile = datasetProfileDao.read(UUID.fromString(id));
			return ResponseEntity.status(HttpStatus.OK).body(profile);
		}
		catch(Exception ex) {
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(null);
		}
	}
	
	@RequestMapping(method = RequestMethod.GET, value = { "/datasetprofile/getAll" })
	public @ResponseBody ResponseEntity<List<DatasetProfile>> getAllDatasetProfiles(){
		
		try {
			List<DatasetProfile> allDatasetProfiles = datasetProfileDao.getAll();
			
			return ResponseEntity.status(HttpStatus.OK).body((allDatasetProfiles));
		}
		catch(Exception ex) {
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.OK).body(null);
		}
		
	}
	
	
	
	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/datasetprofile/create" }, consumes = "application/json", produces="application/json")
	public @ResponseBody ResponseEntity<DatasetProfile> createDatasetProfile(@RequestBody DatasetProfile datasetProfile) {
		
		datasetProfile.setCreated(new Date());
		datasetProfile.setModified(new Date());
		
		DatasetProfileRuleset datasetprofileruleset = datasetProfile.getRuleset();
		DatasetProfileViewstyle datasetprofileviewstyle = datasetProfile.getViewstyle();
		
		if(datasetprofileruleset != null) {
			if(datasetprofileruleset.getId()==null)
				datasetProfileRulesetDao.create(datasetprofileruleset);
			else
				datasetProfileRulesetDao.update(datasetprofileruleset);
			datasetProfile.setRuleset(datasetprofileruleset);
		}
		if(datasetprofileviewstyle != null) {
			if(datasetprofileviewstyle.getId()==null)
				datasetProfileViewstyleDao.create(datasetprofileviewstyle);
			else
				datasetProfileViewstyleDao.update(datasetprofileviewstyle);
			datasetProfile.setViewstyle(datasetprofileviewstyle);
		}
		
		if(datasetProfile.getId()==null)
			datasetProfileDao.create(datasetProfile);
		else
			datasetProfileDao.update(datasetProfile);
		return ResponseEntity.status(HttpStatus.CREATED).body(datasetProfile);
		
	}
	
	
	
	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/datasetprofile/update" }, consumes = "application/json", produces="application/json")
	public @ResponseBody ResponseEntity<DatasetProfile> updateDatasetProfile(@RequestBody DatasetProfile datasetProfile) {
		
		DatasetProfileRuleset datasetprofileruleset = datasetProfile.getRuleset();
		DatasetProfileViewstyle datasetprofileviewstyle = datasetProfile.getViewstyle();
		
		if(datasetprofileruleset != null) {
			if(datasetprofileruleset.getId()==null)
				datasetProfileRulesetDao.create(datasetprofileruleset);
			else
				datasetProfileRulesetDao.update(datasetprofileruleset);
		}
		if(datasetprofileviewstyle != null) {
			if(datasetprofileviewstyle.getId()==null)
				datasetProfileViewstyleDao.create(datasetprofileviewstyle);
			else
				datasetProfileViewstyleDao.update(datasetprofileviewstyle);
		}
		
		if(datasetProfile.getId()==null)
			datasetProfile = datasetProfileDao.create(datasetProfile);
		else
			datasetProfile = datasetProfileDao.update(datasetProfile);
		return ResponseEntity.status(HttpStatus.CREATED).body(datasetProfile);
		
	}
	
	
	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/datasetprofile/delete" }, consumes = "application/json", produces="application/json")
	public @ResponseBody ResponseEntity<DatasetProfile> deleteDatasetProfile(@RequestBody DatasetProfile datasetProfile) {
		
		DatasetProfileRuleset datasetprofileruleset = datasetProfile.getRuleset();
		DatasetProfileViewstyle datasetprofileviewstyle = datasetProfile.getViewstyle();
		
		try {
			if(datasetprofileruleset != null)
				datasetProfileRulesetDao.delete(datasetprofileruleset);
			if(datasetprofileviewstyle != null)
				datasetProfileViewstyleDao.delete(datasetprofileviewstyle);
			datasetProfileDao.delete(datasetProfile);
			return ResponseEntity.status(HttpStatus.OK).body(datasetProfile);
		}
		catch(Exception ex) {
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
		}
		
	}
	
	
	
	
}

