import { Component, ViewChild, OnInit, AfterViewInit, ViewEncapsulation } from "@angular/core";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { Router, ActivatedRoute, Params } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataSource } from "@angular/cdk/table";
import { Observable } from "rxjs/Observable";
import { JsonSerializer } from "../../utilities/JsonSerializer";
import { FormGroup } from "@angular/forms";
import { SnackBarNotificationComponent } from "../../shared/components/notificaiton/snack-bar-notification.component";
import { BaseErrorModel } from "../../models/error/BaseErrorModel";
import { DataManagementPlanService } from "../../services/data-management-plan/data-management-plan.service";
import { DataManagementPlanModel } from "../../models/data-managemnt-plans/DataManagementPlanModel";
import { ServerService } from "../../services/server.service";
import { ExternalSourcesService } from "../../services/external-sources/external-sources.service";



@Component({
  selector: 'app-dmp-editor-component',
  templateUrl: 'dmp-editor.component.html',
  styleUrls: ['./dmp-editor.component.scss'],
  providers: [DataManagementPlanService, ExternalSourcesService],
  encapsulation: ViewEncapsulation.None
})
export class DataManagementPlanEditorComponent implements AfterViewInit {

  isNew = true;
  dataManagementPlan: DataManagementPlanModel;
  formGroup: FormGroup = null;

  filteringOrganisationsAsync: boolean = false;
  filteringResearchersAsync: boolean = false;
  filteredOrganisations: string[];
  filteredResearchers: string[];

  filterOrganisations(value: string): void {
    this.filteredOrganisations = undefined;
    if (value) {
      this.filteringOrganisationsAsync = true;
      
      this.externalSourcesService.searchDMPOrganizations(value).subscribe(organizations => {
        this.filteredOrganisations = organizations;
        this.filteringOrganisationsAsync = false;
      });
    }
  }

  constructor(
    private dataManagementPlanService: DataManagementPlanService,
    private externalSourcesService: ExternalSourcesService,
    private route: ActivatedRoute,
    public snackBar: MatSnackBar,
    public router: Router,
    public language: TranslateService,
  ) {

  }

  ngAfterViewInit() {
    this.route.params.subscribe((params: Params) => {
      const itemId = params['id'];

      if (itemId != null) {
        this.isNew = false;
        this.dataManagementPlanService.getSingle(itemId).map(data => data as DataManagementPlanModel)
          .subscribe(data => {
            this.dataManagementPlan = new JsonSerializer<DataManagementPlanModel>().fromJSONObject(data, DataManagementPlanModel);
            this.formGroup = this.dataManagementPlan.buildForm();
          });
      } else {
        this.dataManagementPlan = new DataManagementPlanModel();
        setTimeout(() => {
          this.formGroup = this.dataManagementPlan.buildForm();
        });
      }
    });
  }

  formSubmit(): void {
    //this.touchAllFormFields(this.formGroup);
    if (!this.isFormValid()) { return; }
    this.onSubmit();
  }

  public isFormValid() {
    return this.formGroup.valid;
  }

  onSubmit(): void {
    this.dataManagementPlanService.createDataManagementPlan(this.formGroup.value).subscribe(
      complete => this.onCallbackSuccess(),
      error => this.onCallbackError(error)
    );
  }

  onCallbackSuccess(): void {
    this.snackBar.openFromComponent(SnackBarNotificationComponent, {
      data: { message: this.isNew ? 'GENERAL.SNACK-BAR.SUCCESSFUL-CREATION' : 'GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE', language: this.language },
      duration: 3000,
      extraClasses: ['snackbar-success']
    })
    this.router.navigate(['/dataManagementPlans']);
  }

  onCallbackError(error: any) {
    this.setErrorModel(error.error);
    //this.validateAllFormFields(this.formGroup);
  }

  public setErrorModel(errorModel: BaseErrorModel) {
    Object.keys(errorModel).forEach(item => {
      (<any>this.dataManagementPlan.errorModel)[item] = (<any>errorModel)[item];
    })
  }

  public cancel(): void {
    this.router.navigate(['/dataManagementPlans']);
  }
}