import { Serializable } from "../Serializable";
import { ValidationContext } from "../../utilities/validators/ValidationContext";
import { FormGroup, FormBuilder, FormControl, Validators } from "@angular/forms";
import { BackendErrorValidator } from "../../utilities/validators/BackendErrorValidator";
import { BaseErrorModel } from "../error/BaseErrorModel";
import { AutoCompleteItem } from "../../shared/components/autocomplete/AutoCompleteItem";

export class DataManagementPlanModel implements Serializable<DataManagementPlanModel> {
	public id: String;
	public label: String;
	public previous: String;
	public version: number;
	public status: String;
	public description: String;
	public selectedProject: AutoCompleteItem;
	// public selectedProjectReference: AutoCompleteItem;
	// public selectedProjectReference: AutoCompleteItem;
	

	public errorModel: BaseErrorModel = new BaseErrorModel();

	fromJSONObject(item: any): DataManagementPlanModel {
		this.id = item.id;
		this.label = item.label;
		this.previous = item.previous;
		this.version = item.version;
		this.status = item.status;
		this.description = item.description;
		
		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }

		const formGroup = new FormBuilder().group({
			label: [{ value: this.label, disabled: disabled }, context.getValidation('label').validators],
			previous: [{ value: this.previous, disabled: disabled }, context.getValidation('previous').validators],
			version: [{ value: this.version, disabled: disabled }, context.getValidation('version').validators],
			status: [{ value: this.status, disabled: disabled }, context.getValidation('status').validators],
			description: [{ value: this.description, disabled: disabled }, context.getValidation('description').validators],
		});

		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'label', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'label')] });
		baseContext.validation.push({ key: 'previous', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'previous')] });
		baseContext.validation.push({ key: 'version', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'version')] });
		baseContext.validation.push({ key: 'status', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'status')] });
		baseContext.validation.push({ key: 'description', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'description')] });

		return baseContext;
	}
}